
package com.tmerclub.cloud.product.mapper;

import com.tmerclub.cloud.common.product.vo.GiveawaySpuVO;
import com.tmerclub.cloud.common.product.vo.GiveawayVO;
import com.tmerclub.cloud.common.product.vo.SpuVO;
import com.tmerclub.cloud.product.dto.GiveawaySpuDTO;
import com.tmerclub.cloud.product.model.GiveawaySpu;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 赠品套装商品项
 *
 * @author tmerclub
 * @date 2022-08-16 11:11:59
 */
public interface GiveawaySpuMapper {

    /**
     * 获取赠品套装商品项列表
     *
     * @return 赠品套装商品项列表
     */
    List<GiveawaySpuVO> list();

    /**
     * 根据商品id获取赠品套装商品项
     *
     * @param spuId 商品id
     * @return 赠品套装商品项
     */
    List<GiveawaySpuVO> getBySpuId(@Param("spuId") Long spuId);

    /**
     * 根据赠品id获取赠品套装商品项
     * @param giveawayId
     * @param lang
     * @param status
     * @return
     */
    List<GiveawaySpuVO> getByGiveawayId(@Param("giveawayId") Long giveawayId, @Param("lang") Integer lang, @Param("status") Integer status);


    /**
     * 根据商品id获取赠品套装商品项
     *
     * @param spuId
     * @return
     */
    GiveawayVO getGiveawayBySpuId(@Param("spuId") Long spuId);


    /**
     * 保存赠品套装商品项
     *
     * @param giveawaySpu 赠品套装商品项
     */
    void save(@Param("giveawaySpu") GiveawaySpu giveawaySpu);

    /**
     * 批量更新赠品套装商品项
     * @param giveawayId
     * @param status
     * @return
     */
    boolean updateStatus(@Param("giveawayId") Long giveawayId, @Param("status") Integer status);

    /**
     * 根据赠品套装商品项id删除赠品套装商品项
     *
     * @param giveawaySpuId
     */
    void deleteById(@Param("giveawaySpuId") Long giveawaySpuId);

    /**
     * 批量保存赠品套装商品项
     *
     * @param giveawaySpuList
     */
    void insertBatch(@Param("giveawaySpuList") List<GiveawaySpuDTO> giveawaySpuList);

    /**
     * 批量删除赠品套餐商品项
     *
     * @param giveawaySpuIds
     */
    void batchDeleteByIds(@Param("giveawaySpuIds") List<Long> giveawaySpuIds);


    /**
     * 获取主赠送商品
     *
     * @param giveawaySpuDTO
     * @param lang
     * @return
     */
    List<SpuVO> getMainSpuPage(@Param("giveawaySpuDTO") GiveawaySpuDTO giveawaySpuDTO, @Param("lang") Integer lang);

    /**
     * 失效赠送商品项状态
     *
     * @param giveawayIdList
     */
    void changeGiveawayActivityStatusByGiveawayIdList(@Param("giveawayIdList") List<Long> giveawayIdList);


}
